@@ -115,7 +115,7 @@ static const uchar *in_buffer_loc(const editbuffer_t *const eb)
 
 static void in_buffer_init(editbuffer_t *eb, 
 			   const uchar * const text, 
-			   const int bypass_initial)
+			   const bool bypass_initial)
 {
     Ginbuf(eb)->ptr = Ginbuf(eb)->buffer = (uchar *)text;
     Ginbuf(eb)->read_count=0;
@@ -400,15 +400,34 @@ static long parsenum(editbuffer_t *eb)
     return ret;
 }
 
-static int parse_next_delta_command(editbuffer_t *eb, struct diffcmd *dc)
+enum edit_op {eof, append, delete, replace};
+
+static enum edit_op parse_next_delta_command(editbuffer_t *eb,
+					     struct diffcmd *dc)
 {
     int cmd;
     long line1, nlines;
 
     cmd = in_buffer_getc(eb);
     if (cmd==EOF)
-	return -1;
+	return eof;
 
+    /*
+     * 1.11 sometines issues replacement text without ops at the front. Example:
+     *
+     * 1.40414
+     * log
+     * @Updating build number tracker
+     * @
+     * text
+     * @#Build Number for ANT. Do not edit!
+     * #Wed Jan 13 07:35:14 CET 2016
+     * build.number=41316
+     * @
+     */
+    if (cmd != 'a' && cmd != 'd')
+	return replace;
+    
     line1 = parsenum(eb);
 
     while (in_buffer_getc(eb) == ' ')
@@ -420,8 +439,8 @@ static int parse_next_delta_command(editbuffer_t *eb, struct diffcmd *dc)
     while (in_buffer_getc(eb) != '\n')
 	;
 
-    if (!nlines || (cmd != 'a' && cmd != 'd') || line1+nlines < line1)
-	fatal_error("Corrupt delta");
+    if (!nlines || line1+nlines < line1)
+	fatal_error("corrupt delta in %s", eb->Gfilename);
 
     if (cmd == 'a') {
 	if (line1 < dc->adprev)
@@ -436,7 +455,7 @@ static int parse_next_delta_command(editbuffer_t *eb, struct diffcmd *dc)
 
     dc->line1 = line1;
     dc->nlines = nlines;
-    return cmd == 'a';
+    return cmd == 'a' ? append : delete;
 }
 
 static void escape_string(editbuffer_t *eb, register char const *s)
@@ -836,12 +855,12 @@ static void process_delta(editbuffer_t *eb,
 			  const enum stringwork func)
 {
     long editline = 0, linecnt = 0, adjust = 0;
-    int editor_command;
+    enum edit_op editor_command;
     struct diffcmd dc;
     uchar *ptr;
 
     eb->Glog = node->patch->log;
-    in_buffer_init(eb, Gnode_text(eb), 1);
+    in_buffer_init(eb, Gnode_text(eb), true);
     eb->Gversion = node->version;
     cvs_number_string(eb->Gversion->number, eb->Gversion_number, sizeof(eb->Gversion_number));
 
@@ -852,16 +871,25 @@ static void process_delta(editbuffer_t *eb,
 	/* coverity[fallthrough] */
     case EDIT:
 	dc.dafter = dc.adprev = 0;
-	while ((editor_command = parse_next_delta_command(eb, &dc)) >= 0) {
-	    if (editor_command) {
+	while ((editor_command = parse_next_delta_command(eb, &dc)) != eof) {
+	    switch (editor_command)
+	    {
+	    case append:
 		editline = dc.line1 + adjust;
 		linecnt = dc.nlines;
 		while (linecnt--)
 		    insertline(eb, editline++, in_get_line(eb));
 		adjust += dc.nlines;
-	    } else {
+		break;
+	    case delete:
 		deletelines(eb, dc.line1 - 1 + adjust, dc.nlines);
 		adjust -= dc.nlines;
+		break;
+	    case replace:
+		fatal_error("edit operation not found in %s", eb->Gfilename);
+	    case eof:
+		/* should never happen */
+		break;
 	    }
 	}
 	break;
@@ -874,22 +902,22 @@ static void expandedit(editbuffer_t *eb)
     editline_t *p, *lim, *l = Gline(eb);
 
     for (p=l, lim=l+Ggap(eb);  p<lim;  ) {
-	in_buffer_init(eb, (*p++).ptr, 0);
+	in_buffer_init(eb, (*p++).ptr, false);
 	expandline(eb);
     }
     for (p+=Ggapsize(eb), lim=l+Glinemax(eb);  p<lim;  ) {
-	in_buffer_init(eb, (*p++).ptr, 0);
+	in_buffer_init(eb, (*p++).ptr, false);
 	expandline(eb);
     }
 #else
     uchar **p, **lim, **l = Gline(eb);
 
     for (p=l, lim=l+Ggap(eb);  p<lim;  ) {
-	in_buffer_init(eb, *p++, 0);
+	in_buffer_init(eb, *p++, false);
 	expandline(eb);
     }
     for (p+=Ggapsize(eb), lim=l+Glinemax(eb);  p<lim;  ) {
-	in_buffer_init(eb, *p++, 0);
+	in_buffer_init(eb, *p++, false);
 	expandline(eb);
     }
 #endif
