@@ -1,26 +1,14 @@
 /*
  *  Copyright © 2006 Keith Packard <keithp@keithp.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or (at
- *  your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful, but
- *  WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along
- *  with this program; if not, write to the Free Software Foundation, Inc.,
- *  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
+ *  SPDX-License-Identifier: GPL-2.0+
  */
 
 /*
  * Build one in-core linked list corresponding to a single CVS
  * master.  Just one entry point, cvs_master_digest(), which takes the
- * structure built by the grammar parse of the master as its single
- * argument.
+ * structure built by the grammar parse of the master as one of its
+ * arguments.
  */
 #include "cvs.h"
 #include "hash.h"
@@ -765,7 +753,7 @@ cvs_master_sort_heads(cvs_master *cm, cvs_file *cvs)
 #endif /* CVSDEBUG */
 }
 
-void
+cvs_commit *
 cvs_master_digest(cvs_file *cvs, cvs_master *cm, rev_master *master)
 /* fill out a linked list capturing the CVS master file structure */
 {
@@ -812,10 +800,12 @@ cvs_master_digest(cvs_file *cvs, cvs_master *cm, rev_master *master)
 		     cvs->gen.master_name);
 #endif /* CVSDEBUG */
     }
-    else
+    else {
 	warn("warning - no master branch generated\n");
+	return NULL;		/* cannot proceed with this master file */
+    }
 #ifdef CVSDEBUG
-w    /*
+    /*
      * Search for other branches
      */
     if (cvs->verbose > 0)
@@ -861,6 +851,7 @@ w    /*
 #endif /* CVSDEBUG */
 
     //rev_list_validate(cm);
+    return trunk;		/* to allow testing for an error in calling function */
 }
 
 // end
