@@ -1,6 +1,8 @@
 """
 Test framework for cvs-fast-export.
 """
+# This code runs correctly under both Python 2 and Python 3.
+# Preserve this property!
 import sys, os, shutil, subprocess, time, filecmp
 
 DEBUG_STEPS    = 1
@@ -12,6 +14,8 @@ verbose = 0
 
 os.putenv("PATH", os.getenv("PATH") + os.pathsep + "..") 
 
+binary_encoding = 'Latin-1'	# Preserves high bits in data
+
 def noisy_run(dcmd, legend=""):
     "Either execute a command or raise a fatal exception."
     if legend:
@@ -40,7 +44,7 @@ def capture_or_die(dcmd, legend=""):
     if verbose >= DEBUG_COMMANDS:
         sys.stdout.write("%s: executing '%s'%s\n" % (caller, dcmd, legend))
     try:
-        return subprocess.Popen(dcmd, shell=True, stdout=subprocess.PIPE).communicate()[0]
+        return subprocess.Popen(dcmd, shell=True, stdout=subprocess.PIPE).communicate()[0].decode(binary_encoding)
     except subprocess.CalledProcessError as e:
         if e.returncode < 0:
             sys.stderr.write("%s: child was terminated by signal %d." % (caller, -e.returncode))
@@ -125,8 +129,9 @@ class RCSRepository:
                 os.system("rm -fr %s" % " ".join(self.conversions))
 
 class CVSRepository(RCSRepository):
-    def __init__(self, name):
+    def __init__(self, name, readonly=False):
         RCSRepository.__init__(self, name)
+        self.readonly = readonly
         self.directory = os.path.join(os.getcwd(), self.name)
         self.checkouts = []
         self.conversions = []
@@ -136,9 +141,13 @@ class CVSRepository(RCSRepository):
             mute = '-Q'
         else:
             mute = ""
-        self.run_with_cleanup("cvs %s -d:local:%s %s" % (mute,
-                                                            self.directory,
-                                                            " ".join(cmd)))
+        if self.readonly:
+            prefix = "CVSREADONLYFS=yes "
+        else:
+            prefix = ""
+        self.run_with_cleanup("%scvs %s -d:local:%s %s" % (prefix, mute,
+                                                           self.directory,
+                                                           " ".join(cmd)))
     def init(self):
         RCSRepository.init(self)
         self.do("init")
@@ -187,7 +196,7 @@ class CVSCheckout:
     def do(self, cmd, *args):
         "Execute a command in the checkout directory."
         with directory_context(self.directory):
-            apply(self.repo.do, [cmd] + list(args))
+            self.repo.do(*([cmd] + list(args)))
     def outdo(self, cmd):
         "Execute a command in the checkout directory."
         with directory_context(self.directory):
@@ -331,11 +340,11 @@ class ConvertComparison:
                 if self.showdiffs:
                     sys.stderr.write(preamble + "common: %d\n" %
                                      len([f for f in gitfiles if f in cvsfiles]))
-                    gitspace_only = {f for f in gitfiles if not f in cvsfiles}
+                    gitspace_only = set([f for f in gitfiles if not f in cvsfiles])
                     if gitspace_only:
                         sys.stderr.write(preamble + "gitspace only: %s\n" %
                                          gitspace_only)
-                    cvs_only = {f for f in cvsfiles if not f in gitfiles}
+                    cvs_only = set([f for f in cvsfiles if not f in gitfiles])
                     if cvs_only:
                         sys.stderr.write(preamble + "CVS only: %s\n" %
                                          cvs_only)
